/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.client;

//---IMPORTS---
import uk.org.iscream.cms.util.*;

/**
 * An object to hold the state of an Alert.
 *
 * When an alert is generated, it has lots of information about
 * the cirscumstances and source of the alert.  This object holds 
 * the information.  It is created by Monitors (specifically in the
 * fireAlert method of MonitorSkeleton).  It then places the alert
 * in the alert queue for Alerters to read and process.
 *
 * @author  $Author: tdb $
 * @version $Id: Alert.java,v 1.20 2004/08/01 10:40:40 tdb Exp $
 */
public class Alert {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public static final String REVISION = "$Revision: 1.20 $";

    /**
     * An internal representation of a NORMAL threshold
     */
    public static final int thresholdNORMAL = 0;
    
    /**
     * An internal representation of a LOWER threshold
     */
    public static final int thresholdLOWER = 1;
    
    /**
     * An internal representation of an UPPER threshold
     */
    public static final int thresholdUPPER = 2;
    
    /**
     * The textual names of the threshold levels.  The position
     * of the names in this array correspond to the internal
     * integer representation constants.
     */
    public static final String[] thresholdLevels = {"NORMAL", "LOWER", "UPPER"};
    
    // note OK and FINAL MUST be present...at the beginning and the end
    
    /**
     * An internal representation of the OK alert
     * This must ALWAYS be 0.
     */
    public static final int alertOK = 0;
    
    /**
     * An internal representation of the NOTICE alert
     */
    public static final int alertNOTICE = 1;
    
    /**
     * An internal representation of the WARNING alert
     */
    public static final int alertWARNING = 2;
    
    /**
     * An internal representation of the CAUTION alert
     */
    public static final int alertCAUTION = 3;
    
    /**
     * An internal representation of the CRITICAL alert
     */
    public static final int alertCRITICAL = 4;
    
    /**
     * An internal representation of the FINAL alert
     * This must ALWAYS be the highest.
     */
    public static final int alertFINAL = 5;
    
    /**
     * The textual names of the alert levels.  The position
     * of the names in this array correspond to the internal
     * integer representation constants.
     */
    public static final String[] alertLevels = {"OK", "NOTICE", "WARNING", "CAUTION", "CRITICAL", "FINAL"};
    
//---STATIC METHODS---

//---CONSTRUCTORS---
    
    /**
     * Construct an Alert object with the appropriate data
     *
     * @param alertLevel the alert level for this Alert object
     * @param lastAlert the last alert level before this one
     * @param thresholdLevel the threshold level that was broken to raise the alert
     * @param source the source of the alert eg, hostname
     * @param thresholdValue the configured value of the treshold level that was broken
     * @param value the value of the attribute that raised the alert
     * @param attributeName the textual representation of the attribute that has raised the alert
     * @param timeTillNextAlert the time until the next alert will be sent concerning this attribute (in seconds)
     * @param initialAlertTime the time the problem with this attribute first occured
     */
    public Alert(int alertLevel, int lastAlert, int thresholdLevel, String source, String thresholdValue, String value, String attributeName, String timeTillNextAlert, long initialAlertTime) {
        _alertLevel = alertLevel;
        _lastAlert = lastAlert;
        _thresholdLevel = thresholdLevel;
        _source = source;
        _thresholdValue = thresholdValue;
        _value = value;
        _attributeName = attributeName;
        _timeTillNextAlert = timeTillNextAlert;
        _initialAlertTime = initialAlertTime;
    }

//---PUBLIC METHODS---
    
    /**
     * Prints a String representation of the Alert object. It
     * is designed to resemble a HashMap.toString(), such that
     * it is similar to the XMLPacket.printAll() method.
     *
     * This method is used by the WebFeeder specifically.
     *
     * @return a String representation of this Alert object.
     */
    public String printAll() {
        String result = "{";
        result += "alertLevel="+_alertLevel+", ";
        result += "lastAlert="+_lastAlert+", ";
        result += "thresholdLevel="+_thresholdLevel+", ";
        result += "source="+_source+", ";
        result += "thresholdValue="+_thresholdValue+", ";
        result += "value="+_value+", ";
        result += "attributeName="+_attributeName+", ";
        result += "timeTillNextAlert="+_timeTillNextAlert+", ";
        result += "initialAlertTime="+_initialAlertTime;
        result += "}";
        return result;
    }

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.FormatName class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---
    
    /**
     * Returns the alert level of this Alert object
     *
     * @return the level
     */
    public int getLevel() {
        return _alertLevel;
    }
    
    /**
     * Returns the previous alert level for the attribute
     *
     * @return the previous level
     */
    public int getLastLevel() {
        return _lastAlert;
    }
    
    /**
     * Returns the threshold level that was broken
     *
     * @return the threshold level
     */
    public int getThreshold() {
        return _thresholdLevel;
    }
    
    /**
     * Returns the source of the alert
     *
     * @return the alerts source eg, hostname
     */
    public String getSource() {
        return _source;
    }
    
    /**
     * Returns the value that the alert level was caused by
     *
     * @return the value
     */
    public String getValue() {
        return _value;
    }
    
    /**
     * Returns the value of the threshold level this alert broke
     *
     * @return the value
     */
    public String getThresholdValue() {
        return _thresholdValue;
    }
    
    /**
     * Returns the textual representation of the name of
     * the attribute that caused the alert.
     *
     * @return the attribute name
     */
    public String getAttributeName() {
        return _attributeName;
    }
    
    /**
     * Returns the time in seconds till the next
     * alert will be sent
     *
     * @return the time till the next alert
     */
    public String getTimeTillNextAlert() {
        return _timeTillNextAlert;
    }
    
    /**
     * Returns the time in milliseconds when
     * the initial alert for this problem with the attribute
     * occured.
     *
     * @return the initial alert time
     */
    public long getInitialAlertTime() {
        return _initialAlertTime;
    }

//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = ClientMain.NAME;
    
    /**
     * The alert level of this Alert object
     */
    private int _alertLevel;
    
    /** 
     * The last alert level for this attribute
     */
    private int _lastAlert;
    
    /**
     * The threshold level that was broken
     */
    private int _thresholdLevel;
    
    /**
     * The source of the alert, eg hostname
     */
    private String _source;
    
    /**
     * The attribute value that caused the alert
     */
    private String _value;
    
    /**
     * The threshold value that was broken
     */
    private String _thresholdValue;
    
    /**
     * The textual representation of the attribute
     * that caused the alert.
     */
    private String _attributeName;
    
    /**
     * The time in seconds till the next alert
     * for this attribute will occur.
     */
    private String _timeTillNextAlert;
    
    /**
     * The time in milliseconds that this
     * alert first occured.
     */
    private long _initialAlertTime;

//---STATIC ATTRIBUTES---

}
