/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.client.monitors;

//---IMPORTS---
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Set;
import java.util.Iterator;
import java.text.NumberFormat;
import uk.org.iscream.cms.server.client.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.componentmanager.*;

/**
 * This Monitor watches the internal server queues.
 * YES! this system is SO great it monitors itself!
 *
 * @author  $Author: tdb $
 * @version $Id: Queue__Monitor.java,v 1.18 2004/08/01 10:40:45 tdb Exp $
 */
public class Queue__Monitor extends MonitorSkeleton {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.18 $";
    
    /**
     * A description of this monitor
     */
    public final String DESC = "Who watches the watchmen?";
    
    private final String SOURCE = "i-scream-server";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

//---PUBLIC METHODS---
    
    /**
     * Analyse a packet of data, and generate an alert if
     * necessary.
     *
     * @param packet the XMLPacket to analyse
     */
    public void analysePacket(XMLPacket packet) {
        if(!checkBooleanConfig("Queue", "Monitor." + _name + ".enable")) {
            return;
        } 
        if (packet.getParam("packet.attributes.type").equals("queueStat")) {
            String source = packet.getParam("packet.attributes.hashCode");
            if (!_hosts.containsKey(source)) {
                _hosts.put(source, new HashMap());
            }
                        
            HashMap queueRegisters = (HashMap) _hosts.get(source);
            
            // get info about the source queue class instance
            String queueName = packet.getParam("packet.attributes.name");
            String maxQueueSize = packet.getParam("packet.queue.attributes.maxSize");
            // unfortunatly we need to check the whole packet
            // to find the queues, and then get the data attributes
            Set packetSet = packet.getSet();
            Iterator i = packetSet.iterator();
            
            while (i.hasNext()) {
                String dataKey = (String) i.next();
                if(dataKey.startsWith("packet.queue.attributes.queue")) {
                    String queueSize = packet.getParam(dataKey);
                    
                    // silently ignore these
                    // they're generated by the queueMonitor for deleted queues
                    if(queueSize.equals("[deleted]")) {
                        break;
                    }

                    // *** now process this queue ***

                    // check if we've seen this queue before on a previous run
                    // if not, we need to create a register for it
                    if(!queueRegisters.containsKey(dataKey)) {
                        queueRegisters.put(dataKey, new Register(_name, _name));
                    }

                    // get the register for this disk
                    Register reg = (Register) queueRegisters.get(dataKey);

                    // get the packet data
                    double qMax, qSize;
                    try {
            
                        if(maxQueueSize==null || queueSize==null) {
                            throw new NumberFormatException("Queue data invalid");
                        }
                        qMax = Double.parseDouble(maxQueueSize);
                        qSize = Double.parseDouble(queueSize);
                    } catch (NumberFormatException e) {
                        _logger.write(this.toString(), Logger.WARNING, "Received packet from "+source+" with bad queue information: "+e);
                        // don't try to continue and process, try next queue
                        break;
                    }
                    
                    boolean useValue = false;
                    try {
                        String option = _cp.getProperty(_name, "Monitor." + _name + ".thresholdMeasure");
                        if (option.equals("VALUE")) {
                            useValue = true;
                        }                            
                    } catch (PropertyNotFoundException e) {
                        // we default to percentage
                    }
                    
                    // this  bit determines if the queue check is a % check
                    // or a literal value check
                    double checkValue;
                    String type;
                    if(useValue) {
                        // queue count
                        checkValue = qSize;
                        type = "count";
                    } else {
                        // % of queue
                        checkValue = (qSize / qMax) * 100;
                        type = "%";
                    }
                    
                    int newThreshold = checkAttributeThreshold(checkValue, reg);
        
                    // format the checkValue to a String
                    NumberFormat nf = NumberFormat.getInstance();
                    nf.setMaximumFractionDigits(2);
                    nf.setMinimumFractionDigits(2);
                    String strCheckValue = nf.format(checkValue);

                    // say which disk had the problem
                    String attributeName = "Size of queue " + type + " in " + queueName + " " + dataKey.substring(dataKey.lastIndexOf('.')+1);
                                            
                    processAlert(newThreshold, attributeName, reg, SOURCE, strCheckValue);
                   
                }
            }
        }
    }

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

    /**
     * return the String representation of what the monitor does
     */
    public String getDescription(){
        return DESC;
    }

//---PRIVATE METHODS---
    
    /**
     * Checks a piece of current data, and returns the
     * threshold it breaches, if any.
     *
     * @param qSize the size of a queue
     * @param reg the Register for the host
     * @return the threshold level breached, if any
     */
    private int checkAttributeThreshold(double qSize, Register reg) {
        for(int thresholdLevel = Alert.thresholdLevels.length - 1; thresholdLevel >= 0; thresholdLevel--) {
            if (reg.getThreshold(thresholdLevel) != -1.0) {
                if(((double) reg.getThreshold(thresholdLevel)) < qSize) {
                    return thresholdLevel;
                }
            }
        }
        return Alert.thresholdNORMAL;
    }

//---ACCESSOR/MUTATOR METHODS---
    
    /**
     * Returns a reference to a specific Queue for this
     * monitor. This Queue returns only the data packets
     * (based on type) that we want too look at.
     *
     * @return a reference to a Queue
     */
    protected Queue getQueue() {
        return MonitorManager.getInstance().getOtherQueue();
    }


//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = "Queue";
    
    /**
     * A reference to the configuration proxy in use
     */
    private ConfigurationProxy _cp = ConfigurationProxy.getInstance();
    
    /**
     * A HashMap of Registers (or groups of Registers), one
     * for each host we're monitoring.
     */
    private HashMap _hosts = new HashMap();

//---STATIC ATTRIBUTES---

}
