/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.clientinterface;

//---IMPORTS---
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.componentmanager.*;
import uk.org.iscream.cms.server.core.*;
import java.net.Socket;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.PrintWriter;
import java.io.InputStreamReader;

/**
 * Acts as a Data Handler to a TCP based client, providing
 * it with a constant stream of XML data for the hosts the
 * client has requested.
 *
 * @author  $Author: tdb $
 * @version $Id: TCPDataHandler.java,v 1.17 2004/08/01 10:40:48 tdb Exp $
 */
class TCPDataHandler extends Thread {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.17 $";
    
//---STATIC METHODS---

//---CONSTRUCTORS---
    
    /**
     * Construct a new TCPDataHandler with a Socket provided
     * by the Control Handler.
     *
     * @param socket The socket to which the Client has connected
     * @throws IOException if something goes wrong, the Control Handler can deal with it
     */
    public TCPDataHandler(Socket socket) throws IOException {
        // set the Thread name
        setName("clientinterface.TCPDataHandler");
        
        ConfigurationProxy cp = ConfigurationProxy.getInstance();
        String configName = "ClientInterface";
        // see if this Queue needs a size limit
        try {
            int queueSizeLimit = Integer.parseInt(cp.getProperty(configName, "Queue.SizeLimit"));
            String queueRemoveAlgorithm = cp.getProperty(configName, "Queue.RemoveAlgorithm");
            int algorithm = StringUtils.getStringPos(queueRemoveAlgorithm, Queue.algorithms);
            if(algorithm != -1) {
                _logger.write(toString(), Logger.DEBUG, "Starting Queue with size limit of "+queueSizeLimit+", using remove algorithm "+queueRemoveAlgorithm);
                // we have valid values, so lets start it.
                _queue = new Queue(queueSizeLimit, algorithm);
            }
            else {
                _logger.write(toString(), Logger.WARNING, "Bad Queue Algorithm configuration, not known: "+queueRemoveAlgorithm);
                // just don't activate a limit
                _queue = new Queue();
            }
            
        } catch (PropertyNotFoundException e) {
            _logger.write(toString(), Logger.DEBUG, "Optional config not set: "+e);
            // just don't activate a limit
            _queue = new Queue();
        } catch (NumberFormatException e) {
            _logger.write(toString(), Logger.WARNING, "Bad Queue SizeLimit configuration: "+e);
            // just don't activate a limit
            _queue = new Queue();
        }
        
        _socket = socket;
        // setup the reader and writer
        _socketIn = new BufferedReader(new InputStreamReader(_socket.getInputStream()));
        _socketOut = new PrintWriter(_socket.getOutputStream(), true);
        _logger.write(toString(), Logger.SYSINIT, "created");
    }
    
//---PUBLIC METHODS---
    /**
     * Main loop for the Data Handler, keeps sending data from
     * it's local Queue.
     */
    public void run() {
        run = true;
        _queueID = _queue.getQueue();
        // run until we get told to stop
        while(run) {
            // get some data, and send some data
            try {
                String xml = (String) _queue.get(_queueID);
                // if we're shutting down and we've been "bumped"
                // it's likely we'll get a null back, and we don't
                // want to send that on to the Client
                if(xml != null) {
                    _socketOut.println(xml);
                }
            }
            catch(InvalidQueueException e) {
                run = false;
                _logger.write(toString(), Logger.ERROR, "Queue failure: "+e);
            }
        }
        // if we get here we've been told to stop
        _logger.write(toString(), Logger.SYSMSG, "Shutting Down");
        // shut down the reader, writer and Socket
        try {
            _socketOut.close();
            _socketIn.close();
            _socket.close();
        }
        catch(IOException e) {
            _logger.write(toString(), Logger.ERROR, "Exception whilst shutting down: "+e);
        }
        // remove ourselves from the queue
        _queue.removeQueue(_queueID);
        _queue.stopMonitor();
    }
    
    /**
     * Method to shutdown this Data Handler. All this actually does
     * is set a flag which the main loop will see and commence shutting
     * down. This method will return before the main loop stops.
     */
    public void shutdown() {
        // set the main loop to stop
        run=false;
        // if the main loop is waiting for data it won't notice the
        // above flag to stop. This bumps it out of the blocked get.
        _queue.releaseQueue(_queueID);
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---
    
    /**
     * Accessor to our Queue. The Control Handler needs to
     * get this reference to register us.
     *
     * @return a reference to our Queue
     */
    public Queue getQueue() {
        return _queue;
    }

//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = ClientInterfaceMain.NAME;
    
    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger = ReferenceManager.getInstance().getLogger();
    
    /** 
     * A hook to the inbound data from the socket
     */
    private BufferedReader _socketIn;
    
    /**
     * A hook to the outbound stream for the socket
     */
    private PrintWriter _socketOut;
    
    /**
     * A reference to the Socket connected to the client
     */
    private Socket _socket;
    
    /**
     * A reference to our Queue
     */
    private Queue _queue;
    
    /**
     * The flag that dictates whether we should be running
     */
    private boolean run;
    
    /**
     * Our queue number within our Queue
     */
    private int _queueID;
    
//---STATIC ATTRIBUTES---

}
